@@ -26,23 +26,17 @@
 #include <KDebug>
 
 #include "PkKCommon.h"
+#include "PkKPolkitClient.h"
 #include "PkKStrings.h"
 #include "PkKBlockGrantAuth.h"
 
-#include <Context>
-#include <ActionButton>
-#include <Auth>
-
-using namespace PolkitQt;
+using namespace PolKitQt;
 
 namespace PolkitKde
 {
 
-PkKAction::PkKAction(QWidget *parent)
-        : QWidget(parent),
-          m_pfe(0),
-          m_updatingEntry(false),
-          m_mostRecentUpdateCount(0)
+PkKAction::PkKAction(PolKitContext* cxt, QWidget *parent)
+        : QWidget(parent), m_context(cxt), m_pfe(0)
 {
     setupUi(this);
 
@@ -51,8 +45,6 @@ PkKAction::PkKAction(QWidget *parent)
 
     blockPB->setIcon(KIcon("object-locked"));
     grantPB->setIcon(KIcon("dialog-ok"));
-    revertPB->setIcon(KIcon("document-revert"));
-    revokePB->setIcon(KIcon("list-remove"));
 
     connect(vendorUrlKUL, SIGNAL(leftClickedUrl(const QString &)),
             SLOT(openUrl(const QString &)));
@@ -69,44 +61,39 @@ PkKAction::PkKAction(QWidget *parent)
     m_explicitModel = new ExplicitAuthorizationsModel(explicitAuthTV);
     explicitAuthTV->setModel(m_explicitModel);
 
-    m_authdb = polkit_context_get_authorization_db(Context::instance()->getPolKitContext());
+    connect(showChB, SIGNAL(stateChanged(int)), this, SLOT(updateEntryView()));
+
+    DBusError dbus_error;
+    DBusConnection *con;
+
+    dbus_error_init(&dbus_error);
+    con = dbus_bus_get(DBUS_BUS_SYSTEM, &dbus_error);
+
+    m_pkTracker = polkit_tracker_new();
+    polkit_tracker_set_system_bus_connection(m_pkTracker, con);
+    polkit_tracker_init(m_pkTracker);
+    m_explicitModel->setPkTracker(m_pkTracker);
+
+    m_pkRevokeAction = polkit_action_new();
+    polkit_action_set_action_id(m_pkRevokeAction, AUTH_ACTION_REVOKE);
 
     connect(explicitAuthTV, SIGNAL(clicked(const QModelIndex &)),
             this, SLOT(explicitAuthClicked(const QModelIndex &)));
 
     // Using the new polkit-qt lib
-    m_pkModifyBt = new ActionButton(modifyPB, AUTH_ACTION_MODIFY_DEFAULTS, this);
-    m_pkModifyBt->setText(i18n("&Modify"));
-    m_pkModifyBt->setIcon(KIcon("object-locked"));
-    m_pkModifyBt->setYesIcon(KIcon("security-high"));
-    m_pkModifyBt->setNoIcon(KIcon("security-low"));
-    m_pkModifyBt->setAuthIcon(KIcon("object-locked-verified"));
-
-    m_pkReadChB = new ActionButton(showChB, AUTH_ACTION_READ, this);
-    m_pkReadChB->setText(i18n("&Show authorizations from all users"));
-    m_pkReadChB->setIcon(KIcon("object-locked"));
-    m_pkReadChB->setYesIcon(KIcon("security-high"));
-    m_pkReadChB->setNoIcon(KIcon("security-low"));
-    m_pkReadChB->setAuthIcon(KIcon("object-locked-verified"));
-    connect(m_pkReadChB, SIGNAL(clicked(QAbstractButton *, bool)),
-            m_pkReadChB, SLOT(activate()));
-    connect(m_pkReadChB, SIGNAL(activated()),
-            this, SLOT(updateExplicitModel()));
-
-    m_pkRevokeAct = new Action(AUTH_ACTION_REVOKE, this);
-    connect(m_pkRevokeAct, SIGNAL(dataChanged()),
-            this, SLOT(revokeChanged()));
+//     m_modifyAction = new PkAction(AUTH_ACTION_MODIFY_DEFAULTS, winId(), QIcon(), i18n("Modify"), "Toool tip tap",  this);
+//     connect(modifyPB, SIGNAL(clicked()), m_modifyAction, SLOT(activate()));
+//     connect(m_modifyAction, SIGNAL(activated()), this, SLOT(modifyPB_clicked()));
+    connect(modifyPB, SIGNAL(clicked()), this, SLOT(modifyPB_clicked()));
 }
 
 PkKAction::~PkKAction()
 {
-    if (m_pfe != 0) {
+    if (m_pfe != 0)
         polkit_policy_file_entry_unref(m_pfe);
-    }
 
-    if (m_authdb) {
-        polkit_authorization_db_unref(m_authdb);
-    }
+    polkit_tracker_unref(m_pkTracker);
+    polkit_action_unref(m_pkRevokeAction);
 }
 
 polkit_bool_t fillAuthModel(PolKitAuthorizationDB* authdb, PolKitAuthorization* auth, void* user_data)
@@ -120,25 +107,17 @@ polkit_bool_t fillAuthModel(PolKitAuthorizationDB* authdb, PolKitAuthorization*
 
 void PkKAction::updateEntryView()
 {
-    QString actionId = polkit_policy_file_entry_get_id(m_pfe);
-    // check to see if we are modifying an entrie
-    // if so we should wait
-    if (m_updatingEntry && m_oldActionId == actionId) {
-        // this is not beautiful but it's the best
-        // way to avoid conflictings updates
-        return;
-    }
-
     // the pfe is just arriving
-    m_pkModifyBt->setMasterEnabled(false);
+    modifyPB->setEnabled(false);
     revokePB->setEnabled(false);
-    m_oldActionId       = actionId;
     //Action description
-    QString description = QString::fromUtf8(polkit_policy_file_entry_get_action_description(m_pfe));
+    QString actionId    = polkit_policy_file_entry_get_id(m_pfe);
+    QString description = polkit_policy_file_entry_get_action_description(m_pfe);
     QString iconName    = polkit_policy_file_entry_get_action_icon_name(m_pfe);
-    QString vendor      = QString::fromLocal8Bit(polkit_policy_file_entry_get_action_vendor(m_pfe));
-    QString vendorUrl   = QString::fromLocal8Bit(polkit_policy_file_entry_get_action_vendor_url(m_pfe));
+    QString vendor      = polkit_policy_file_entry_get_action_vendor(m_pfe);
+    QString vendorUrl   = polkit_policy_file_entry_get_action_vendor_url(m_pfe);
 
+//     identifierL->setText(actionId);
     descriptionL->setText(description);
 
     if (vendorUrl.isEmpty()) {
@@ -180,23 +159,20 @@ void PkKAction::updateEntryView()
     for (int i = 0; i < results.size(); i++) {
         result = results.at(i);
         // Activate or deactivating bold
-        if (result != m_factoryAllowAny) {
+        if (result != m_factoryAllowAny)
             allowAnyCB->setItemData(i, m_fontBold, Qt::FontRole);
-        } else {
+        else
             allowAnyCB->setItemData(i, m_fontNotBold, Qt::FontRole);
-        }
 
-        if (result != m_factoryAllowInactive) {
+        if (result != m_factoryAllowInactive)
             allowInactiveCB->setItemData(i, m_fontBold, Qt::FontRole);
-        } else {
+        else
             allowInactiveCB->setItemData(i, m_fontNotBold, Qt::FontRole);
-        }
 
-        if (result != m_factoryAllowActive) {
+        if (result != m_factoryAllowActive)
             allowActiveCB->setItemData(i, m_fontBold, Qt::FontRole);
-        } else {
+        else
             allowActiveCB->setItemData(i, m_fontNotBold, Qt::FontRole);
-        }
     }
 
     allowAnyCB->setCurrentIndex(allowAnyCB->findData(m_defaultsAllowAny));
@@ -204,37 +180,22 @@ void PkKAction::updateEntryView()
     allowActiveCB->setCurrentIndex(allowActiveCB->findData(m_defaultsAllowActive));
 
     // Explicit Authorizations
-    updateExplicitModel();
-}
-
-void PkKAction::updateExplicitModel()
-{
-    PolKitError  *err;
-    PolKitAction *action;
-    PolKitCaller *self;
-    DBusError    dbus_error;
 
+    m_authdb = polkit_context_get_authorization_db(m_context);
+    PolKitError* err;
     m_explicitModel->clear();
+    PolKitAction* action = polkit_action_new_from_string_representation(actionId.toUtf8());
+    PolKitCaller *self;
+    DBusError dbus_error;
     dbus_error_init(&dbus_error);
-    action = polkit_action_new_from_string_representation(polkit_policy_file_entry_get_id(m_pfe));
-    self   = polkit_tracker_get_caller_from_pid(Context::instance()->getPolKitTracker(),
-                                                QCoreApplication::applicationPid(),
-                                                &dbus_error);
+    self = polkit_tracker_get_caller_from_pid(m_pkTracker, getpid(), &dbus_error);
 
     if (m_authdb && action && self != NULL) {
-        if (showChB->isChecked()) {
-            polkit_authorization_db_foreach_for_action(m_authdb,
-                                                       action,
-                                                       fillAuthModel,
-                                                       m_explicitModel,
-                                                       &err);
+        if (showChB->checkState() == Qt::Checked && PkKPolkitClient::instance()->getAuth("org.freedesktop.policykit.read")) {
+            polkit_authorization_db_foreach_for_action(m_authdb, action, fillAuthModel, m_explicitModel, &err);
         } else {
-            polkit_authorization_db_foreach_for_action_for_uid(m_authdb,
-                                                               action,
-                                                               getuid(),
-                                                               fillAuthModel,
-                                                               m_explicitModel,
-                                                               &err);
+            showChB->setCheckState(Qt::Unchecked);
+            polkit_authorization_db_foreach_for_action_for_uid(m_authdb, action, getuid(), fillAuthModel, m_explicitModel, &err);
         }
     }
     explicitAuthTV->resizeColumnToContents(0);
@@ -242,20 +203,20 @@ void PkKAction::updateExplicitModel()
     explicitAuthTV->resizeColumnToContents(2);
     explicitAuthTV->resizeColumnToContents(3);
     explicitAuthTV->resizeColumnToContents(4);
-}
 
-void PkKAction::revokeChanged()
-{
-    QModelIndexList modelIndexL = explicitAuthTV->selectionModel()->selectedRows();
-    if (modelIndexL.size()) {
-        explicitAuthClicked(modelIndexL.at(0));
-    }
+    //polkit_authorization_db_unref(authdb);
+    //polkit_action_unref(action);
 }
 
 void PkKAction::explicitAuthClicked(const QModelIndex &index)
 {
     PolKitAuthorization *auth = m_explicitModel->authEntry(index);
     if (auth) {
+        revokePB->setEnabled(true);
+//TODO this does not work as expected
+//like if we cannot revoke an action and it is
+//because impicit is no the button should be disable
+//else explicit the bt should hide
         uid_t our_uid;
         uid_t for_uid;
         uid_t pimp_uid;
@@ -266,9 +227,10 @@ void PkKAction::explicitAuthClicked(const QModelIndex &index)
 
         for_uid = polkit_authorization_get_uid(auth);
 
-        // we need org.fd.polkit.revoke if:
-        // 1) the auth is for another user than us
-        // 2) the auth was granted by someone else than us
+        /* we need org.fd.polkit.revoke if:
+         *  1) the auth is for another user than us
+         *  2) the auth was granted by someone else than us
+         */
         need_revoke = false;
         if (for_uid != our_uid) {
             need_revoke = true;
@@ -279,25 +241,26 @@ void PkKAction::explicitAuthClicked(const QModelIndex &index)
         }
 
         if (need_revoke) {
-            m_pkRevokeAct->setPolkitAction(AUTH_ACTION_REVOKE);
-        } else {
-            m_pkRevokeAct->setPolkitAction();
-        }
-        revokePB->setEnabled(m_pkRevokeAct->isEnabled());
-    } else {
+            if (m_pkRevokeAction && polkit_authorization_db_is_uid_blocked_by_self(m_authdb,
+                    m_pkRevokeAction,
+                    getuid(),
+                    NULL))
+                revokePB->setEnabled(true);
+        } else
+            revokePB->setEnabled(true);
+    } else
         revokePB->setEnabled(false);
-    }
 }
 
 void PkKAction::on_grantPB_clicked()
 {
-    PkKBlockGrantAuth *frm = new PkKBlockGrantAuth(m_pfe, false, this);
+    PkKBlockGrantAuth *frm = new PkKBlockGrantAuth(m_context, m_pfe, false, this);
     frm->show();
 }
 
 void PkKAction::on_blockPB_clicked()
 {
-    PkKBlockGrantAuth *frm = new PkKBlockGrantAuth(m_pfe, true, this);
+    PkKBlockGrantAuth *frm = new PkKBlockGrantAuth(m_context, m_pfe, true, this);
     frm->exec();
     delete frm;
 }
@@ -309,23 +272,49 @@ void PkKAction::on_revokePB_clicked()
     PolKitAuthorization *auth;
 
     if (modelIndexL.size() && (auth = m_explicitModel->authEntry(modelIndexL.at(0)))) {
-        polkit_authorization_ref(auth);
-        if (!m_pkRevokeAct->activate(winId())) {
-            return;
+        PolKitAuthorizationDB *authdb;
+        PolKitError *pk_error = NULL;
+
+        authdb = polkit_context_get_authorization_db(m_context);
+
+        uid_t our_uid;
+        uid_t for_uid;
+        uid_t pimp_uid;
+        bool need_revoke;
+        polkit_bool_t is_negative;
+
+        our_uid = getuid();
+
+        for_uid = polkit_authorization_get_uid(auth);
+
+        /* we need org.fd.polkit.revoke if:
+         *  1) the auth is for another user than us
+         *  2) the auth was granted by someone else than us
+         */
+        need_revoke = false;
+        if (for_uid != our_uid) {
+            need_revoke = true;
+        } else if (polkit_authorization_was_granted_explicitly(auth, &pimp_uid, &is_negative)) {
+            if (pimp_uid != our_uid) {
+                need_revoke = true;
+            }
         }
 
-        PolKitAuthorizationDB *authdb;
-        PolKitError *pk_error;
+        if (need_revoke) {
+            // TODO there was a case where a click on cancel
+            // AND without the "!" made it pass and the revoke worked so
+            // more testing is needed..
+            if (!PkKPolkitClient::instance()->getAuth(AUTH_ACTION_REVOKE)) {
+                return;
+            }
+        }
 
-        authdb = polkit_context_get_authorization_db(Context::instance()->getPolKitContext());
-
-        pk_error = NULL;
         if (!polkit_authorization_db_revoke_entry(authdb, auth, &pk_error)) {
-            kDebug() << "Error doing revoke: " << polkit_error_get_error_name(pk_error)
-                     << ": "                   << polkit_error_get_error_message(pk_error);
+            kDebug() << "Error doing revoke: " << polkit_error_get_error_code(pk_error)
+            << ": "                   << polkit_error_get_error_name(pk_error)
+            << ": "                   << polkit_error_get_error_message(pk_error);
             polkit_error_free(pk_error);
         }
-        polkit_authorization_unref(auth);
     }
 }
 
@@ -341,10 +330,8 @@ void PkKAction::setPolKitPolicyFileEntry(PolKitPolicyFileEntry *pfe)
 
 void PkKAction::on_allowAnyCB_currentIndexChanged(int index)
 {
-    if (!m_pfe) {
+    if (!m_pfe)
         return;
-    }
-
     if (allowAnyCB->itemData(index) == m_factoryAllowAny) {
         allowAnyCB->setFont(m_fontNotBold);
         allowAnyL->setFont(m_fontNotBold);
@@ -358,10 +345,8 @@ void PkKAction::on_allowAnyCB_currentIndexChanged(int index)
 
 void PkKAction::on_allowInactiveCB_currentIndexChanged(int index)
 {
-    if (!m_pfe) {
+    if (!m_pfe)
         return;
-    }
-
     if (allowInactiveCB->itemData(index) == m_factoryAllowInactive) {
         allowInactiveCB->setFont(m_fontNotBold);
         allowInactiveL->setFont(m_fontNotBold);
@@ -375,10 +360,8 @@ void PkKAction::on_allowInactiveCB_currentIndexChanged(int index)
 
 void PkKAction::on_allowActiveCB_currentIndexChanged(int index)
 {
-    if (!m_pfe) {
+    if (!m_pfe)
         return;
-    }
-
     if (allowActiveCB->itemData(index) == m_factoryAllowActive) {
         allowActiveCB->setFont(m_fontNotBold);
         allowActiveL->setFont(m_fontNotBold);
@@ -394,49 +377,47 @@ void PkKAction::checkDefaults()
 {
     if (polkit_policy_default_equals(m_defaults, m_factory)) {
         // IF they are equal we still need to check if the user has the defaults
-        if (allowAnyCB->itemData(allowAnyCB->currentIndex())           == m_factoryAllowAny &&
-            allowInactiveCB->itemData(allowInactiveCB->currentIndex()) == m_factoryAllowInactive &&
-            allowActiveCB->itemData(allowActiveCB->currentIndex())     == m_factoryAllowActive)
-        {
+        if (allowAnyCB->itemData(allowAnyCB->currentIndex()) == m_factoryAllowAny &&
+                allowInactiveCB->itemData(allowInactiveCB->currentIndex()) == m_factoryAllowInactive &&
+                allowActiveCB->itemData(allowActiveCB->currentIndex()) == m_factoryAllowActive)
+
             revertPB->setEnabled(false);
-        } else {
+        else
             revertPB->setEnabled(true);
-        }
-    } else {
+    } else
         revertPB->setEnabled(true);
-    }
 }
 
 bool PkKAction::checkModified()
 {
     // IF they are equal we still need to check if the user has the defaults
-    if (allowAnyCB->itemData(allowAnyCB->currentIndex())           == m_defaultsAllowAny &&
-        allowInactiveCB->itemData(allowInactiveCB->currentIndex()) == m_defaultsAllowInactive &&
-        allowActiveCB->itemData(allowActiveCB->currentIndex())     == m_defaultsAllowActive)
-    {
-        m_pkModifyBt->setMasterEnabled(false);
-    } else {
-        m_pkModifyBt->setMasterEnabled(true);
-    }
+    if (allowAnyCB->itemData(allowAnyCB->currentIndex()) == m_defaultsAllowAny &&
+            allowInactiveCB->itemData(allowInactiveCB->currentIndex()) == m_defaultsAllowInactive &&
+            allowActiveCB->itemData(allowActiveCB->currentIndex()) == m_defaultsAllowActive)
+
+        modifyPB->setEnabled(false);
+    else
+        modifyPB->setEnabled(true);
     return modifyPB->isEnabled();
 }
 
-void PkKAction::on_modifyPB_clicked()
+void PkKAction::modifyPB_clicked()
 {
-    m_updatingEntry = true;
-    PolKitError *pk_error;
-    PolKitPolicyDefault *new_defaults;
-    // we cache the pfe as this is all asyncronous
-    PolKitPolicyFileEntry *pfe = m_pfe;
-    polkit_policy_file_entry_ref(pfe);
-    kDebug() << "Setting PFE";
+kDebug();
+// computeAndObtainAuth
 
-    PolKitResult allowAny      = (PolKitResult) allowAnyCB->itemData(allowAnyCB->currentIndex()).toInt();
-    PolKitResult allowInactive = (PolKitResult) allowInactiveCB->itemData(allowInactiveCB->currentIndex()).toInt();
-    PolKitResult allowActive   = (PolKitResult) allowActiveCB->itemData(allowActiveCB->currentIndex()).toInt();
+// PkKPolkitClient::instance()->getAuth(AUTH_ACTION_MODIFY_DEFAULTS)
+// PkAction action(QIcon(), "Punch!", "org.gnome.policykit.examples.punch", "Punch it!", winId(), this);
+
+    if (PkAuth::computeAndObtainAuth(AUTH_ACTION_MODIFY_DEFAULTS, winId())) {
+    kDebug();
+        PolKitError *pk_error;
+        PolKitPolicyDefault *new_defaults;
+
+        PolKitResult allowAny      = (PolKitResult) allowAnyCB->itemData(allowAnyCB->currentIndex()).toInt();
+        PolKitResult allowInactive = (PolKitResult) allowInactiveCB->itemData(allowInactiveCB->currentIndex()).toInt();
+        PolKitResult allowActive   = (PolKitResult) allowActiveCB->itemData(allowActiveCB->currentIndex()).toInt();
 
-    // manually call activate as this method is reentrant
-    if (m_pkModifyBt->activate()) {
         new_defaults = polkit_policy_default_new();
         polkit_policy_default_set_allow_any(new_defaults, allowAny);
         polkit_policy_default_set_allow_inactive(new_defaults, allowInactive);
@@ -444,7 +425,7 @@ void PkKAction::on_modifyPB_clicked()
 
         pk_error = NULL;
         polkit_policy_file_entry_debug(m_pfe);
-        if (!polkit_policy_file_entry_set_default(pfe, new_defaults, &pk_error)) {
+        if (!polkit_policy_file_entry_set_default(m_pfe, new_defaults, &pk_error)) {
             kDebug() << "Error: code=" << polkit_error_get_error_code(pk_error)
             << ": " << polkit_error_get_error_name(pk_error)
             << ": " << polkit_error_get_error_message(pk_error);
@@ -453,14 +434,11 @@ void PkKAction::on_modifyPB_clicked()
 
         polkit_policy_default_unref(new_defaults);
     }
-    m_updatingEntry = false;
-    polkit_policy_file_entry_unref(pfe);
+    kDebug() << "----------------------------------------------";
 }
 
 void PkKAction::on_revertPB_clicked()
 {
-    PolKitPolicyFileEntry *pfe = m_pfe;
-    polkit_policy_file_entry_ref(pfe);
     if (polkit_policy_default_equals(m_defaults, m_factory)) {
         // IF They are equal but we got clicked
         // is because the user changed the values
@@ -468,17 +446,16 @@ void PkKAction::on_revertPB_clicked()
         allowAnyCB->setCurrentIndex(allowAnyCB->findData(m_defaultsAllowAny));
         allowInactiveCB->setCurrentIndex(allowInactiveCB->findData(m_defaultsAllowInactive));
         allowActiveCB->setCurrentIndex(allowActiveCB->findData(m_defaultsAllowActive));
-    } else if (Auth::computeAndObtainAuth(AUTH_ACTION_MODIFY_DEFAULTS, winId())) {
+    } else if (PkKPolkitClient::instance()->getAuth(AUTH_ACTION_MODIFY_DEFAULTS)) {
         // if auth is fine just try to change
         PolKitError *pk_error = NULL;
-        if (!polkit_policy_file_entry_set_default(pfe, m_factory, &pk_error)) {
+        if (!polkit_policy_file_entry_set_default(m_pfe, m_factory, &pk_error)) {
             kDebug() << "Error: code=" << polkit_error_get_error_code(pk_error)
             << ": "           << polkit_error_get_error_name(pk_error)
             << ": "           << polkit_error_get_error_message(pk_error);
             polkit_error_free(pk_error);
         }
     }
-    polkit_policy_file_entry_unref(pfe);
 }
 
 void PkKAction::openUrl(const QString &url)
@@ -488,8 +465,7 @@ void PkKAction::openUrl(const QString &url)
 
 QList<PolKitResult> PkKAction::resultsOrdered()
 {
-    return QList<PolKitResult>()
-           << POLKIT_RESULT_NO
+    return QList<PolKitResult>() << POLKIT_RESULT_NO
            << POLKIT_RESULT_ONLY_VIA_ADMIN_AUTH_ONE_SHOT
            << POLKIT_RESULT_ONLY_VIA_ADMIN_AUTH
            << POLKIT_RESULT_ONLY_VIA_ADMIN_AUTH_KEEP_SESSION
