@@ -24,7 +24,6 @@
 #include "AuthDialog.h"
 
 #include <QProcess>
-#include <QPainter>
 
 #include <KDebug>
 
@@ -32,8 +31,7 @@
 #include <KUser>
 
 AuthDialog::AuthDialog(PolKitPolicyFileEntry *entry, uint pid)
-        : KDialog(0, Qt::Dialog | Qt::CustomizeWindowHint),
-          m_entry(entry)
+        : KDialog(0, Qt::Dialog | Qt::WindowStaysOnTopHint | Qt::CustomizeWindowHint)
 {
     setupUi(mainWidget());
     // the dialog needs to be modal to darken the parent window
@@ -41,7 +39,7 @@ AuthDialog::AuthDialog(PolKitPolicyFileEntry *entry, uint pid)
     setButtons(Ok | Cancel | Details);
 
     kDebug() << "Getting action message...";
-    QString actionMessage = QString::fromUtf8(polkit_policy_file_entry_get_action_message(entry));
+    QString actionMessage = QString::fromLocal8Bit(polkit_policy_file_entry_get_action_message(entry));
     if (actionMessage.isEmpty()) {
         kWarning() << "Could not get action message for action.";
         lblHeader->hide();
@@ -51,34 +49,11 @@ AuthDialog::AuthDialog(PolKitPolicyFileEntry *entry, uint pid)
         setCaption(actionMessage);
     }
 
-    // loads the standard key icon
-    QPixmap icon = KIconLoader::global()->loadIcon("dialog-password",
-                                                   KIconLoader::NoGroup,
-                                                   KIconLoader::SizeHuge,
-                                                   KIconLoader::DefaultState);
-    // create a paiter to paint the action icon over the key icon
-    QPainter painter(&icon);
-    const int iconSize = icon.size().width();
-    // the the emblem icon to size 32
-    int overlaySize = 32;
-    // try to load the action icon
-    const QPixmap pixmap = KIconLoader::global()->loadIcon(polkit_policy_file_entry_get_action_icon_name(entry),
-                                                           KIconLoader::NoGroup,
-                                                           overlaySize,
-                                                           KIconLoader::DefaultState,
-                                                           QStringList(),
-                                                           0,
-                                                           true);
-    // if we're able to load the action icon paint it over the
-    // key icon.
-    if (!pixmap.isNull()) {
-        QPoint startPoint;
-        // bottom right corner
-        startPoint = QPoint(iconSize - overlaySize - 2,
-                            iconSize - overlaySize - 2);
-        painter.drawPixmap(startPoint, pixmap);
-    }
-
+    QPixmap icon = KIconLoader::global()->loadIcon(polkit_policy_file_entry_get_action_icon_name(entry),
+                   KIconLoader::NoGroup, KIconLoader::SizeHuge, KIconLoader::DefaultState, QStringList(), NULL, true);
+    if (icon.isNull())
+        icon = KIconLoader::global()->loadIcon("dialog-password",
+                                               KIconLoader::NoGroup, KIconLoader::SizeHuge);
     setWindowIcon(icon);
     lblPixmap->setPixmap(icon);
 
@@ -88,7 +63,7 @@ AuthDialog::AuthDialog(PolKitPolicyFileEntry *entry, uint pid)
         AuthDetails *details = new AuthDetails(entry, i18nc("Unknown action", "Unknown"), this);
         setDetailsWidget(details);
     } else {
-        m_appname = QString::fromUtf8(tmp);
+        m_appname = QString::fromLocal8Bit(tmp);
         AuthDetails *details = new AuthDetails(entry, m_appname, this);
         setDetailsWidget(details);
     }
@@ -97,13 +72,7 @@ AuthDialog::AuthDialog(PolKitPolicyFileEntry *entry, uint pid)
     lePassword->setFocus();
 
     errorMessageKTW->hide();
-
-    m_userModelSIM = new QStandardItemModel(this);
-    m_userModelSIM->setSortRole(Qt::UserRole);
-    userCB->setModel(m_userModelSIM);
-
-    connect(userCB, SIGNAL(currentIndexChanged(int)),
-            this, SLOT(on_userCB_currentIndexChanged(int)));
+    connect(userCB, SIGNAL(currentIndexChanged(int)), this, SLOT(on_userCB_currentIndexChanged(int)));
 }
 
 AuthDialog::~AuthDialog()
@@ -119,28 +88,24 @@ void AuthDialog::accept()
 void AuthDialog::setRequest(const QString &request, bool requiresAdmin)
 {
     kDebug() << request;
-    QVariant userLogin;
-    userLogin = m_userModelSIM->data(
-                        m_userModelSIM->index(userCB->currentIndex(), 0), Qt::UserRole);
-    if (request.startsWith(QLatin1String("password:"), Qt::CaseInsensitive)) {
+    if (request.startsWith("password:", Qt::CaseInsensitive)) {
         if (requiresAdmin) {
-            if (userLogin.isNull()) {
-                lblPassword->setText(i18n("Password for root:"));
-            } else {
+            if (!userCB->itemData(userCB->currentIndex()).isNull()) {
                 lblPassword->setText(i18n("Password for %1:",
-                                          userLogin.toString()));
+                                          userCB->itemData(userCB->currentIndex()).toString()));
+            } else {
+                lblPassword->setText(i18n("Password for root:"));
             }
         } else {
             lblPassword->setText(i18n("Password:"));
         }
-    } else if (request.startsWith(QLatin1String("password or swipe finger:"),
-                                                        Qt::CaseInsensitive)) {
+    } else if (request.startsWith("password or swipe finger:", Qt::CaseInsensitive)) {
         if (requiresAdmin) {
-            if (userLogin.isNull()) {
-                lblPassword->setText(i18n("Password or swipe finger for root:"));
-            } else {
+            if (!userCB->itemData(userCB->currentIndex()).isNull()) {
                 lblPassword->setText(i18n("Password or swipe finger for %1:",
-                                          userLogin.toString()));
+                                          userCB->itemData(userCB->currentIndex()).toString()));
+            } else {
+                lblPassword->setText(i18n("Password or swipe finger for root:"));
             }
         } else {
             lblPassword->setText(i18n("Password or swipe finger:"));
@@ -151,43 +116,23 @@ void AuthDialog::setRequest(const QString &request, bool requiresAdmin)
 
 }
 
-void AuthDialog::setOptions(PolicyKitKDE::KeepPassword keep, bool requiresAdmin, const QStringList &adminUsers)
+void AuthDialog::setOptions(KeepPassword keep, bool requiresAdmin, const QStringList &adminUsers)
 {
     switch (keep) {
-        case PolicyKitKDE::KeepPasswordNo:
-            cbRemember->hide();
-            cbSessionOnly->hide();
-            break;
-        case PolicyKitKDE::KeepPasswordSession:
-            cbRemember->setText(i18n("Remember authorization for this session"));
-            cbRemember->show();
-            cbSessionOnly->hide();;
-            break;
-        case PolicyKitKDE::KeepPasswordAlways:
-            cbRemember->setText(i18n("Remember authorization"));
-            cbRemember->show();
-            cbSessionOnly->show();
-            break;
-    }
-
-    // Check the blacklist to grab if user unchecked the "remember authorization" check box
-    PolicyKitKDE::KeepPassword keepPassword;
-    keepPassword = PolicyKitKDE::readDefaultKeepPassword(polkit_policy_file_entry_get_id(m_entry), keep);
-    if (keepPassword != keep) {
-        switch (keepPassword) {
-            case PolicyKitKDE::KeepPasswordNo:
-                cbRemember->setChecked(false);
-                cbSessionOnly->setChecked(false);
-                break;
-            case PolicyKitKDE::KeepPasswordSession:
-                cbRemember->setChecked(true);
-                cbSessionOnly->setChecked(true);
-                break;
-            case PolicyKitKDE::KeepPasswordAlways:
-                cbRemember->setChecked(true);
-                cbSessionOnly->setChecked(false);
-                break;
-        }
+    case KeepPasswordNo:
+        cbRemember->hide();
+        cbSessionOnly->hide();
+        break;
+    case KeepPasswordSession:
+        cbRemember->setText(i18n("Remember authorization for this session"));
+        cbRemember->show();
+        cbSessionOnly->hide();;
+        break;
+    case KeepPasswordAlways:
+        cbRemember->setText(i18n("Remember authorization"));
+        cbRemember->show();
+        cbSessionOnly->show();
+        break;
     }
 
     if (requiresAdmin) {
@@ -230,13 +175,11 @@ void AuthDialog::createUserCB(const QStringList &adminUsers)
         * wrong password and the dialog is recycled)
         */
     if (adminUsers.count() && (userCB->count() - 1) != adminUsers.count()) {
-        // Clears the combobox in the case some user be added
-        m_userModelSIM->clear();
+        // Clears the combobox as some user might be added
+        userCB->clear();
 
         // Adds a Dummy user
-        QStandardItem *selectItem;
-        m_userModelSIM->appendRow(selectItem = new QStandardItem(i18n("Select user")));
-        selectItem->setSelectable(false);
+        userCB->addItem(i18n("Select user"));
 
         // For each user
         foreach(const QString &adminUser, adminUsers) {
@@ -249,25 +192,19 @@ void AuthDialog::createUserCB(const QStringList &adminUsers)
 
             // Display user Full Name IF available
             QString display;
-            if (!user.property(KUser::FullName).toString().isEmpty()) {
+            if (!user.property(KUser::FullName).toString().isEmpty())
                 display = user.property(KUser::FullName).toString() + " (" + user.loginName() + ')';
-            } else {
+            else
                 display = user.loginName();
-            }
-
-            QStandardItem *item = new QStandardItem(display);
-            // DO NOT store UID as polkit get's confused in the case whether
-            // you have another user with the same ID (ie root)
-            item->setData(user.loginName(), Qt::UserRole);
 
             // load user icon face
-            if (!user.faceIconPath().isEmpty()) {
-                item->setIcon(KIcon(user.faceIconPath()));
+            if (user.faceIconPath().isEmpty()) {
+                // DO NOT store UID as polkit get's confused in the case whether
+                // you have another user with the same ID (ie root)
+                userCB->addItem(display, user.loginName());
             } else {
-                item->setIcon(KIcon("user-identity"));
+                userCB->addItem(KIcon(user.faceIconPath()), display, user.loginName());
             }
-            // appends the user item
-            m_userModelSIM->appendRow(item);
         }
 
         // Show the widget and set focus
@@ -278,24 +215,17 @@ void AuthDialog::createUserCB(const QStringList &adminUsers)
 
 QString AuthDialog::adminUserSelected() const
 {
-    QVariant userLogin;
-    userLogin = m_userModelSIM->data(
-                        m_userModelSIM->index(userCB->currentIndex(), 0), Qt::UserRole);
-    if (userLogin.isNull()) {
+    if (userCB->itemData(userCB->currentIndex()).isNull())
         return QString();
-    } else {
-        return userLogin.toString();
-    }
+    else
+        return userCB->itemData(userCB->currentIndex()).toString();
 }
 
 QString AuthDialog::selectCurrentAdminUser()
 {
     KUser currentUser;
     for (int i = 1; i < userCB->count(); i++) {
-        QVariant userLogin;
-        userLogin = m_userModelSIM->data(
-                        m_userModelSIM->index(i, 0), Qt::UserRole);
-        if (userLogin.toString() == currentUser.loginName()) {
+        if (userCB->itemData(i).toString() == currentUser.loginName()) {
             userCB->setCurrentIndex(i);
             return currentUser.loginName();
         }
@@ -305,11 +235,8 @@ QString AuthDialog::selectCurrentAdminUser()
 
 void AuthDialog::on_userCB_currentIndexChanged(int index)
 {
-    QVariant userLogin;
-    userLogin = m_userModelSIM->data(
-                        m_userModelSIM->index(index, 0), Qt::UserRole);
     // itemData is Null when "Select user" is selected
-    if (userLogin.isNull()) {
+    if (userCB->itemData(index).isNull()) {
         lePassword->setEnabled(false);
         lblPassword->setEnabled(false);
         cbRemember->setEnabled(false);
@@ -352,23 +279,16 @@ void AuthDialog::setPasswordShowChars(bool showChars)
         lePassword->setEchoMode(QLineEdit::Password);
 }
 
-PolicyKitKDE::KeepPassword AuthDialog::keepPassword() const
+KeepPassword AuthDialog::keepPassword() const
 {
-    if (cbRemember->isHidden()) {
-        // cannot make it keep
-        return PolicyKitKDE::KeepPasswordNo;
-    }
-    if (cbSessionOnly->isHidden()) {
-        // can keep only for session
-        return cbRemember->isChecked() ? PolicyKitKDE::KeepPasswordSession :
-                                         PolicyKitKDE::KeepPasswordNo;
-    }
+    if (cbRemember->isHidden()) // cannot make it keep
+        return KeepPasswordNo;
+    if (cbSessionOnly->isHidden()) // can keep only for session
+        return cbRemember->isChecked() ? KeepPasswordSession : KeepPasswordNo;
     // can keep either way
-    if (cbRemember->isChecked()) {
-        return cbSessionOnly->isChecked() ? PolicyKitKDE::KeepPasswordSession :
-                                            PolicyKitKDE::KeepPasswordAlways;
-    }
-    return PolicyKitKDE::KeepPasswordNo;
+    if (cbRemember->isChecked())
+        return cbSessionOnly->isChecked() ? KeepPasswordSession : KeepPasswordAlways;
+    return KeepPasswordNo;
 }
 
 AuthDetails::AuthDetails(PolKitPolicyFileEntry *entry, const QString &appname, QWidget *parent)
@@ -378,13 +298,13 @@ AuthDetails::AuthDetails(PolKitPolicyFileEntry *entry, const QString &appname, Q
 
     app_label->setText(appname);
 
-    action_label->setText(QString::fromUtf8(polkit_policy_file_entry_get_action_description(entry)));
+    action_label->setText(polkit_policy_file_entry_get_action_description(entry));
     QString actionId = polkit_policy_file_entry_get_id(entry);
     action_label->setTipText(i18n("Click to edit %1", actionId));
     action_label->setUrl(actionId);
 
-    QString vendor    = QString::fromUtf8(polkit_policy_file_entry_get_action_vendor(entry));
-    QString vendorUrl = QString::fromUtf8(polkit_policy_file_entry_get_action_vendor_url(entry));
+    QString vendor    = polkit_policy_file_entry_get_action_vendor(entry);
+    QString vendorUrl = polkit_policy_file_entry_get_action_vendor_url(entry);
     if (!vendor.isEmpty()) {
         vendorUL->setText(vendor);
         vendorUL->setTipText(i18n("Click to open %1", vendorUrl));
